﻿using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.Pdf.Interactive;
using Syncfusion.Pdf.Parsing;
using Syncfusion.Pdf.Security;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Security.Cryptography;
using System.Security.Cryptography.Pkcs;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ConsoleApplication1
{
    class Program
    {
        public static string DataPathBase
        {
            get
            {

                return System.Environment.CurrentDirectory + @"\..\..\Data\";

            }
        }
        public static string DataPathOutput
        {
            get
            {
                if (!System.IO.Directory.Exists(System.Environment.CurrentDirectory + @"\..\..\Output\"))
                    System.IO.Directory.CreateDirectory(System.Environment.CurrentDirectory + @"\..\..\Output\");
                return System.Environment.CurrentDirectory + @"\..\..\Output\";
            }
        }
        static void Main(string[] args)
        {
            PdfLoadedDocument loadedDocument = new PdfLoadedDocument(DataPathBase + @"Input.pdf");
            PdfLoadedPage page = loadedDocument.Pages[0] as PdfLoadedPage;
            //Create a new PDF signature instance.  
            PdfSignature signature = new PdfSignature(loadedDocument, page, null, "Sign1");
            //Set the signature bounds.  
            signature.Bounds = new RectangleF(0, 0, 200, 100);
            //Set allow signature field
            signature.DocumentPermissions = PdfCertificationFlags.AllowFormFill | PdfCertificationFlags.AllowComments | PdfCertificationFlags.ForbidChanges;
            //Call the compute hash event.  
            signature.ComputeHash += Signature_ComputeHash;


            PdfSignatureField sigField = new PdfSignatureField(page, "SignatureField");
            sigField.Bounds = new RectangleF(100, 100, 200, 100);
            loadedDocument.Form.Fields.Add(sigField);
            loadedDocument.Save(DataPathOutput + "Output2.pdf");
            loadedDocument.Close(true);

            PdfLoadedDocument loadedDocument2 = new PdfLoadedDocument(DataPathOutput + "Output2.pdf");
            PdfLoadedSignatureField loadedSignField = loadedDocument2.Form.Fields[1] as PdfLoadedSignatureField;

            PdfCertificate cert = new PdfCertificate(DataPathBase + "PDF.pfx", "syncfusion");
             PdfSignature signature2 = new PdfSignature(loadedDocument2, loadedDocument2.Pages[0], cert, "Sign2", loadedSignField);
            signature2.DocumentPermissions = PdfCertificationFlags.AllowFormFill | PdfCertificationFlags.AllowComments | PdfCertificationFlags.ForbidChanges;

            loadedSignField.Signature = signature2;

            loadedDocument2.Save(DataPathOutput + "OutputFinal2.pdf");
            loadedDocument2.Close(true);

            

            System.Diagnostics.Process.Start(System.IO.Path.Combine(DataPathOutput, "OutputFinal2.pdf"));
        }

        private static void Signature_ComputeHash(object sender, PdfSignatureEventArgs ars)
        {
            //Get the document bytes. 
            byte[] documentBytes = ars.Data;
           
            X509Certificate2 certificate = new X509Certificate2(DataPathBase+"PDF.pfx", "syncfusion");

            //Include the signed data to PDF.  
            ars.SignedData = Sign(documentBytes, certificate);
        }
        public static byte[] Sign(byte[] data, X509Certificate2 certificate)
        {
            if (data == null)
                throw new ArgumentNullException("data");
            if (certificate == null)
                throw new ArgumentNullException("certificate");

            // setup the data to sign 
            ContentInfo content = new ContentInfo(data);
            SignedCms signedCms = new SignedCms(content, false);
            CmsSigner signer = new CmsSigner(certificate);
    
            // create the signature
            signedCms.ComputeSignature(signer);
  
            return signedCms.Encode();
        }
    }
}
